#!/usr/bin/perl
######################################################## 
# $Id: $
#  
# Copyright (C) 2009  Elden Crom
# 
# This file is part of paparazzi.
# 
# paparazzi is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# paparazzi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with paparazzi; see the file COPYING.  If not, write to
# the Free Software Foundation, 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA. 
######################################################## 


###########################
# 
#Questions:
#a) Where does the plotter put the KML file that it generates? (does it work?, I can't seem to find the file.)
#b) If it works could somebody send me one of their logs?  (I want to know what it displays.)
#c) I started writing this perl script before I realized that the logger already does something with KML.....
#      Attached is my script (still a work in progress) and a sample of its output.
#      you can rerun it with 
#     'perl /shared/e/pap_ubun/utm_geo_convertion.pl  09_12_10__07_03_14.data -scale_plane=8.0 -min_move=5'
#c1) Eventually, I'm going to make a server out of it, for use in real time as well as post processing.
#    where should it go in the source tree (or is perl banned? :-)
#d) What value in the data file is the carrot? (grep for $showCarrot, on the cmd line -showCarrot=-10 )
#     I thought it was the "NAVIGATION" message but it shows me strange results,
#     all the carrots are excatly on the previous a/c location...to good to be true... 
#e) Is anybody good with coordinate frame translation in 3d space? 
#   I want to show the rudder and alerions, etc at their command positions.
#   (a DH transform matrix? I think, maybe?, http://en.wikipedia.org/wiki/Denavit-Hartenberg_Parameters )
#   see  sub DebuggingRotation show that my xforms are wrong in 'sub convertCoordinateFrame'


my $pn = uc($0);#procname
$pn =~ s/.PL$//;
$pn =~ s#^.*[\\/]##;
sub usage {print "
#############################################################################
#                                                                         
# DESCRIPTION:                                                            
#   Command line utility to convert paparazzi *.data files, so that they  
#   can be displayed in Google Earth (5.1)
#                                                                         
# USAGE:
#   perl $0  <[*.data]+> 
#             [-out=*.kml]
#             [-start_time=0]
#             [-stop_time=1e10]
#             [-min_move=2.5]
#             [-scale_plane=5.0]
#             [-showCarrot=10]
#             [-help]
#
#   command line options can be in any order and take precedence over
#   env vars.
#
#   -out=              --- the output file name, '-' to go to std out
#   -start_time=0      --- the time into the flight to begin generating tracks
#   -stop_time=1e10    --- the time into the flight to stop generating tracks
#   -min_move=2.5      --- don't generate track data if until movement beyond min_move has occurred 
#                          (ie don't put a 1000 points at the same location since it's sitting still on the workbench) 
#   -scale_plane=5.0   --- magnify the size of the plane (the default plane is 55in by 42in)
#   -plane_model=p.dae --- magnify the size of the plane (the default plane is 55in by 42in)
#   -help              --- this help screen
#   
# EXAMPLES:                                                             
#   perl $0 ~/downloads/09_11_27__17_41_31.data -start_time=300 -stop_time=520 -out=-
#        --- generate kml flight tracks from 300 seconds to 520 seconds and write the output to stdout
#
#   $0 09_11_27__17_41_31.data
#        --- generate kml flight tracks for the entire file, write the output to 09_11_27__17_41_31.kml
#
#   $0 --help      
#      prints this help
#                                                                         
# ENVIRONMENTAL VARIABLES: (same as command line options, the defaults are shown)
#   set ${pn}__START_TIME=0                                          
#   set ${pn}__STOP_TIME=1e300
#   set ${pn}__MIN_MOVE=2.5
#   set ${pn}__SCALE_PLANE=5.0
#   set ${pn}__PLANE_MODEL=models/SimplePlane.dae
#                                                                       
# Initail: Elden Crom -- Nov  30,2009
# Updated:
#############################################################################
# Hit return to exit ########################################################
#############################################################################
";<STDIN>;exit(1);
}
#To use the Geo::Coordinates::UTM module first you must install it, see below
eval {#use the require and import instead of 'use Geo::Coordinates::UTM;' 
      #so that we can print an error message
      require Geo::Coordinates::UTM;
      import  Geo::Coordinates::UTM;
      1;
} || die("
Cannot load Geo::Coordinates::UTM -- please execute 
  perl -MCPAN -e 'install Geo::Coordinates::UTM'
or 
  ##goto www.google.com search for 'perl Geo::Coordinates::UTM site:cpan.org' and get the .tgz file
  firefox http://search.cpan.org/~grahamc/Geo-Coordinates-UTM-0.07/ #and download
  tar xzf Geo-Coordinates-UTM-0.08 ; cd Geo-Coordinates-UTM-0.08/
  perl Makefile.PL ; make ; make test ; make install
before running this program
     ");
use Getopt::Long qw(GetOptions HelpMessage);
use File::Basename;
use File::Spec;
use Cwd;
use Sys::Hostname;
use strict;
##### set up the global variables
my ($startTime,$stopTime,$minMov);
my ($scale_plane,$plane_model);
my $date=getTime();
my $gen_comment=   "    generated on ".$date." by"
                ."\n $0 ".join(" ",@ARGV).""
                ."\n from in dir ".getcwd.""
                ."\n on host ".hostname.""
                ."\n by ".$ENV{"USER"}." (".(getlogin || getpwuid($<)).")"
                ."\n ";
my $showCarrot=undef;
parse_command_line_and_execute();
exit();
################################################################################################
################## DDDDDDDDDDDDDDDDOOOOOOOONNNNNNNNNNNEEEEEEEEEEEEE ############################
################################################################################################

################################################################################################
sub parse_command_line_and_execute {
   if(@ARGV<1){usage();}
   ####### set up the defaults..........
   $startTime      = $ENV{"${pn}__START_TIME"              } || 0;
   $stopTime       = $ENV{"${pn}__STOP_TIME"               } || 1e300;
   $minMov         = $ENV{"${pn}__MIN_MOVE"                } || 2.5;
   $scale_plane    = $ENV{"${pn}__SCALE_PLANE"             } || 5;
   $plane_model    = $ENV{"${pn}__PLANE_MODEL"             } || "models/SimplePlane.dae";
   
   my $outputFilename = "";
   my $help           = 0;
   if(not GetOptions( "start_time=i"   , \$startTime
                     ,"stop_time=i"    , \$stopTime
                     ,"out=s"          , \$outputFilename
                     ,"min_move=f"     , \$minMov
                     ,"scale_plane=f"  , \$scale_plane
                     ,"plane_model=s"  , \$plane_model
                     ,"showCarrot=f"   , \$showCarrot
                     ,"help"           , \$help
              )){usage();};
   if($help){usage();}
   my @dir_file=@ARGV;
   if($outputFilename ne ""){
      if(scalar @dir_file > 1 ){
         print "Converting multiple files if the outfile name is supported yet!";
         usage();
      }else{
         convertFile($dir_file[0],$outputFilename);
      }
   #}elsif (1==0){###need to adjust the input file if this is to be used as a server..
   #
   }else{
      for (@dir_file) {recurse_dir_or_file($_);}
   }
}
##################################################################
sub recurse_dir_or_file {
  for my $ff (@_){
     if(-d $ff ){
        #opendir OD,"$ff" or die("Cannot open directory '$ff'");
        #my @f= grep /\.data$/ ,(grep !/(^\.$)/, (grep !/(^\.\.$)/, readdir OD));
        #close(OD);
        #for my $x (@f) {recurse_dir_or_file(File::Spec->catfile($ff,$x));}
        print "sorry, there appears to be an error in perl v5.10.0 you must specify each file\n";
     }elsif(-f $ff){convertFileCreateOutputFilename("".$ff."");
     }else{print "I don't know what a '$ff' is.\n";}  
  }
}
##################################################################
sub getTime{
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
   $wday=qw/Sun Mon Tus Wed Thu Fri/[$wday];   
   return sprintf("%d_%02d_%02d__%02d_%02d_%02d__$wday",($year+1900),($mon+1),$mday,$hour,$min,$sec);
}
##################################################################
sub convertFileCreateOutputFilename {
   my ($inputfile,undef)=(@_);
   my $outputFilename=$inputfile;
   $outputFilename =~s/.data//;
   $outputFilename.=".kml";
   convertFile($inputfile,$outputFilename);
}
##################################################################
sub convertFile {
   ############ expect
   my ($inputfile,$outputFilename)=(@_);

   print "inputfile='$inputfile'\n";
   print "outputFilename='$outputFilename'\n";

   my ($IN,$OUT);
   open($IN ,"<$inputfile"     ) || die "unable to open '$inputfile'";
   open($OUT,">$outputFilename") || die "unable to open '$outputFilename'";
   #if   ($outputFilename eq "-") {open($OUT,STDOUT            ) || die "unable to open stdout";}
   #else{                          open($OUT,">$outputFilename") || die "unable to open '$outputFilename'";}

   writeModel($outputFilename);

   my($easting_lv, $northing_lv, $alt_lv,$course_lv,$moved_lv,$z_lv)=(0,0,0,0,0);
   my($min_easting ,$max_easting,
      $min_northing,$max_northing,
      $min_alt     ,$max_alt);
   my $cords="";
   my $planeTilt="";
   my $ppz_mode_lv="-1";
   my $leg_start=0;
   my $i=1;
   my $time=0;
   my $name = $outputFilename;
   my ($heading,$tilt,$roll)=(0,0,0);
   my ($IR_SENSORS_cnt,$IR_SENSORS__GPS_cnt_lv)=(0,0);
   my $GPS_cnt=0;
   my $PPRZ_MODE_cnt=0;
   my $ellipsoid="WGS-84";
   my $marks="";
   my $elevator_lv=0;
   $name =~ s#.*\/##;   
   $name =~ s#.kml##;
   my($lat_lv,$lon_lv);
   my $lines="";
   my $actuators="";
   my $rudder_lv=0;
   my $alerion_lv=0;
   my %h;
   my($nav_utm_east,$nav_utm_north,$nav_utm_zone);
   my $nav_cords="";
   my $NAVIGATION_REF_cnt=0;
   while(<$IN>){   
      my ($tm,$t1,$LineName,@data)=split;
      $h{$LineName}++;
      $time=$tm+0.00;
      next if ($time < $startTime);
      next if ($time > $stopTime);
      #print "\$LineName='$LineName' data=".join(",",@data)."\n";
      if("GPS" eq $LineName){
         my ($mode,$utm_east,$utm_north,
             $course,$alt,$speed,$climb,$itow,$utm_zone,$gps_nb_err,$blank)=@data;
         my($z, $easting, $northing) = ($utm_zone,($utm_east/100.0) ,($utm_north/100.0));
         $alt=$alt/100.0+10;
         $moved_lv = sqrt( ($easting -$easting_lv)**2 + ($northing-$northing_lv)**2 + ($alt-$alt_lv)**2); 
         next if ( $moved_lv < $minMov);
         #######if the GPS has not located use yet then ignore the data...
         next if ($mode  == 0);
         $easting_lv =$easting ;
         $northing_lv=$northing;
         $northing_lv=$northing;
         $z=$z."S";###TODO:the zone 'S' needs to be estimated
         #print "\$ellipsoid, \$z, \$easting, \$northing=$ellipsoid, $z, $easting, $northing\n";
         my ($lat, $lon) = utm_to_latlon($ellipsoid, $z, $easting, $northing);
         #print "\$lat, \$lon=$lat,$lon\n";
         if($GPS_cnt==0){
              $marks.=placeMarkPlane("Flight_Launch","msn_airports_launch",$lat,$lon,$alt,"1");
              ($min_easting ,$max_easting )=($easting,$easting);
              ($min_northing,$max_northing)=($northing,$northing);
              ($min_alt     ,$max_alt     )=($alt,$alt);
         }
         if($min_easting  > $easting ){$min_easting  = $easting; }
         if($min_northing > $northing){$min_northing = $northing;}
         if($min_alt      > $alt     ){$min_alt      = $alt;     }
         if($max_easting  < $easting ){$max_easting  = $easting; }
         if($max_northing < $northing){$max_northing = $northing;}
         if($max_alt      < $alt     ){$max_alt      = $alt;     }
         $z_lv=$z;
         $cords.="$lon,$lat,$alt\n";
         $lat_lv=$lat;
         $lon_lv=$lon;
         $alt_lv=$alt;
         $course_lv=($course/10.0)-0.0;
         $GPS_cnt++;
      }elsif ("PPRZ_MODE" eq $LineName){
         next if $GPS_cnt ==0;
         my ($ap_mode,$ap_gaz,$ap_lateral,
             $ap_horizontal,$if_calib_mode,$mcu1_status)=@data;
         if($ppz_mode_lv eq "-1"){$ppz_mode_lv=$ap_mode; next ;} ####ignore first entry 
         next if($ppz_mode_lv eq $ap_mode);  ### ignore it if we have not changed
         ######we have changed mode and need to a record a new color 
         my $duration=$time-$leg_start;
         $leg_start=$tm;   
         my $description="ppzmode=$ppz_mode_lv, \nduration=".sprintf("%.3f",$duration)." seconds";       
         $lines.=makeLeg("leg $i",$description,getStyle($ppz_mode_lv),$cords,$nav_cords,"$planeTilt$actuators");
         $i++;
         $cords="";
         $nav_cords="";
         $planeTilt="";
         $actuators="";
         $ppz_mode_lv=$ap_mode;
         $PPRZ_MODE_cnt++;
      }elsif ("IR_SENSORS" eq $LineName){
         next if  $GPS_cnt==0;
         #next if  $moved_lv < 1;
         #next if ($GPS_cnt==$IR_SENSORS__GPS_cnt_lv);
         $IR_SENSORS__GPS_cnt_lv=$GPS_cnt;
         my($ir1,$ir2,$longitudinal,$lateral,$vertical)=@data;
         my($est_deg_roll_per_tick)=(0.13);
         my($est_deg_tilt_per_tick)=(0.13);
         $tilt=-$longitudinal*$est_deg_tilt_per_tick;
         $roll=-$lateral*$est_deg_roll_per_tick;;
         #if($IR_SENSORS_cnt%1==0){
            $planeTilt.=makeModelTilt($lon_lv,$lat_lv,$alt_lv,$course_lv,$tilt ,$roll ,$scale_plane,$plane_model);

            $actuators.=makeModelTilt($lon_lv,$lat_lv,$alt_lv,$course_lv,$tilt ,$roll ,$scale_plane,"models/T.dae");
            #my ($course2,$tilt2,$roll2)=convertCoordinateFrame($elevator_lv,$alerion_lv,$rudder_lv,$course_lv,$tilt,$roll);
            #$actuators.=makeModelTilt($lon_lv,$lat_lv,$alt_lv,$course2  ,$tilt2,$roll2,$scale_plane,"models/T.dae");
         #}
         #if($IR_SENSORS_cnt>10){die;}
         $IR_SENSORS_cnt++;
      }elsif("ACTUATORS" eq $LineName){
         next if $GPS_cnt ==0;
         #$actuators.=makeModelTilt($lon_lv,$lat_lv,$alt_lv,$course_lv,$tilt,$roll,$scale_plane);
         my($throttle,$blank,$elevator,$aleronL,$aleronR,$rudder)=split(",",$data[0]);
         $elevator_lv=getTilt($elevator,1200,1800);
         $rudder_lv  =getTilt($rudder  ,1200,1800);
         $alerion_lv =getTilt($aleronL ,1200,1800); # assume that $aleronL and $aleronR are the same
      }elsif("ATTITUDE" eq $LineName){
         next if $GPS_cnt ==0;
         my($phi,$psi,$theta)=@data;
         #$elevator_lv=getTilt($elevator,1200,1800);
         #$rudder_lv=getTilt($rudder,1200,1800);
         #$alerion_lv=getTilt($aleronL,1200,1800); # assume that $aleronL and $aleronR are the same
      }elsif("DESIRED" eq $LineName){
         next if $GPS_cnt ==0;
         #my($roll,$pitch,$course,$x,$y,$altitude,$climb)=@data;
      }elsif("BLOCK" eq $LineName){
         next if $GPS_cnt ==0; ###?? is this generated from the GSE or from the NAVIGATION command
         #it does not appear in the messages.h file
         my($cur_block, $cur_stage)=@data;
         #record the point and start a new block
         my $duration=$time-$leg_start;
         $leg_start=$tm;   
         my $description="ppzmode=$ppz_mode_lv, \nduration=".sprintf("%.3f",$duration)." seconds\n".
                         "block=$cur_block,stage=$cur_stage"; 
         my $title="leg $i,$cur_block,$cur_stage";
         $lines.=makeLeg($title,$description,getStyle($ppz_mode_lv),$cords,$nav_cords,"$planeTilt$actuators");
         $marks.=placeMarkPlane($title,"msn_airports_land",$lat_lv,$lon_lv,$alt_lv,"1");
         $i++;
         $cords="";
         $nav_cords="";
         $planeTilt="";
         $actuators="";
      }elsif("NAVIGATION" eq $LineName){
         next if $GPS_cnt ==0;
         next if $NAVIGATION_REF_cnt ==0;
         my($cur_block, $cur_stage, $pos_x, $pos_y, $dist2_wp, $dist2_home)=@data;

         my($easting, $northing) = (($nav_utm_east+$pos_x) ,($nav_utm_north+$pos_y));
         my ($lat, $lon) = utm_to_latlon($ellipsoid, $z_lv, $easting, $northing);
         #############################3 TURN THIS BACK ON!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
         #############################3 TURN THIS BACK ON!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
         #############################3 TURN THIS BACK ON!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
         if($showCarrot!=undef){
            $nav_cords.="$lon_lv,$lat_lv,$alt_lv\n";
            $nav_cords.="$lon,$lat,".($alt_lv+$showCarrot)."\n";
            $nav_cords.="$lon_lv,$lat_lv,$alt_lv\n";
         }
         #############################3 TURN THIS BACK ON!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
         #############################3 TURN THIS BACK ON!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      }elsif("NAVIGATION_REF" eq $LineName){
         next if $GPS_cnt ==0;
         ($nav_utm_east,$nav_utm_north,$nav_utm_zone)=@data;
         #$nav_utm_east /=100.0; nope these are in meters appearantly...not centimeters
         #$nav_utm_north/=100.0;
         $NAVIGATION_REF_cnt++;
      }


   }
   if($GPS_cnt==0 or $PPRZ_MODE_cnt==0){
      print "File '$inputfile' missing data GPS_cnt=$GPS_cnt PPRZ_MODE_cnt=$PPRZ_MODE_cnt IR_SENSORS_cnt=$IR_SENSORS_cnt\n"
   }elsif(length($cords) > 1){
       my $duration=$time-$leg_start;
       my $description="ppzmode=$ppz_mode_lv, \nduration=".sprintf("%.3f",$duration)." seconds";       
       $lines.=makeLeg("final leg $i",$description,getStyle($ppz_mode_lv),$cords,$nav_cords,"$planeTilt$actuators");
   }

   my $bb=makeBoundingBox($ellipsoid, $z_lv,
       $min_easting-10  , $max_easting+10  ,
       $min_northing-10 , $max_northing+10 ,
       $min_alt         , $max_alt+10     
   );

   #$marks.=placeMarkPlane("Elden's House","msn_airports_land",32.432074,-110.971943,864,"1");
   $marks.=placeMarkPlane("Flight_Land","msn_airports_land",$lat_lv,$lon_lv,$alt_lv,"1");

   ######## Now write the output file#####
   header($OUT,"Flight_$name","generatedFrom:$inputfile,$gen_comment");
   my $folders="";


   ################ TODO remove the if when done debugging the actuators frame of reference problem
   my $vis="0";
   if(1==1){
      $folders.=getFolder("FlightLegs"       ,$lines    ,$vis,"");
      #$folders.=getFolder("Actuators"        ,$actuators,$vis,"");
      $folders.=getFolder("Walls"            ,$bb       ,$vis,"");
      $folders.=getFolder("Various Markers"  ,$marks    ,$vis,"");
      #$folders.=getFolder("A/C Tilt"         ,$planeTilt,$vis,"");
   }


   my $dbg_rot=DebuggingRotation($ellipsoid, $z_lv,$min_easting,$min_northing,$min_alt+10);
   $folders.=getFolder("fourNorth A/C Tilt"         ,$dbg_rot,"1","");


   print $OUT $folders;
   footer($OUT);
   for (keys %h) {print sprintf "%20d %s\n",$h{$_}, $_;} # | sort
}


#####################################################################################################
#####################################################################################################
#####################################################################################################
######################## Begin Debug: Trying to figure out how to do the rotation!!!!!! #############
#####################################################################################################
#####################################################################################################
#####################################################################################################
sub DebuggingRotation {
   my ($ellipsoid, $z,$easting,$northing,$alt)=@_;
   my ($lat, $lon) = utm_to_latlon($ellipsoid, $z, $easting, $northing);
   my $actuators;
   my($course1 ,$tilt1 ,$roll1)=(0,0,0);
   my($course2 ,$tilt2 ,$roll2)=(0,0,20);
   #$actuators.=makeModelTilt($lon,$lat,$alt, $course1  ,$tilt1, $roll1, 100,"models/T.dae");
   #$actuators.=makeModelTilt($lon,$lat,$alt, $course2  ,$tilt2, $roll2, 100,"models/T.dae");
   #
   #$northing-=50;
   #($lat, $lon) = utm_to_latlon($ellipsoid, $z, $easting, $northing);
   #($course1 ,$tilt1 ,$roll1)=(0,0,0);
   #($course2 ,$tilt2 ,$roll2)=($course1 ,$tilt1+20 ,$roll1);
   #$actuators.=makeModelTilt($lon,$lat,$alt, $course1  ,$tilt1, $roll1, 100,"models/T.dae");
   #$actuators.=makeModelTilt($lon,$lat,$alt, $course2  ,$tilt2, $roll2, 100,"models/T.dae");
   #
   #$northing-=50;
   #($lat, $lon) = utm_to_latlon($ellipsoid, $z, $easting, $northing);
   #($course1 ,$tilt1 ,$roll1)=(0,0,0);
   #($course2 ,$tilt2 ,$roll2)=($course1+20 ,$tilt1 ,$roll1);
   #$actuators.=makeModelTilt($lon,$lat,$alt, $course1  ,$tilt1, $roll1, 100,"models/T.dae");
   #$actuators.=makeModelTilt($lon,$lat,$alt, $course2  ,$tilt2, $roll2, 100,"models/T.dae");

   $easting-=50;
   $northing+=50;
   ($lat, $lon) = utm_to_latlon($ellipsoid, $z, $easting, $northing);
   ($course1 ,$tilt1 ,$roll1)=(0,0,0);
   ($course2 ,$tilt2 ,$roll2)=convertCoordinateFrame(0,0,20,$course1,$tilt1 ,$roll1);
   $actuators.=makeModelTilt($lon,$lat,$alt, $course1  ,$tilt1, $roll1, 100,"models/T.dae");
   $actuators.=makeModelTilt($lon,$lat,$alt, $course2  ,$tilt2, $roll2, 100,"models/T.dae");

   $northing+=50;
   ($lat, $lon) = utm_to_latlon($ellipsoid, $z, $easting, $northing);
   ($course1 ,$tilt1 ,$roll1)=(0,0,0);
   ($course2 ,$tilt2 ,$roll2)=convertCoordinateFrame(0,20,0,$course1,$tilt1 ,$roll1);
   $actuators.=makeModelTilt($lon,$lat,$alt, $course1  ,$tilt1, $roll1, 100,"models/T.dae");
   $actuators.=makeModelTilt($lon,$lat,$alt, $course2  ,$tilt2, $roll2, 100,"models/T.dae");

   $northing+=50;
   ($lat, $lon) = utm_to_latlon($ellipsoid, $z, $easting, $northing);
   ($course1 ,$tilt1 ,$roll1)=(0,0,0);
   ($course2 ,$tilt2 ,$roll2)=convertCoordinateFrame(20,0,0,$course1,$tilt1 ,$roll1);
   $actuators.=makeModelTilt($lon,$lat,$alt, $course1  ,$tilt1, $roll1, 100,"models/T.dae");
   $actuators.=makeModelTilt($lon,$lat,$alt, $course2  ,$tilt2, $roll2, 100,"models/T.dae");

   print "Done with pass throughs\n\n\n\n";

   $alt+=50;

   $easting-=50;
   ($lat, $lon) = utm_to_latlon($ellipsoid, $z, $easting, $northing);
   ($course1 ,$tilt1 ,$roll1)=(90,0,0);
   ($course2 ,$tilt2 ,$roll2)=convertCoordinateFrame(20,0,0,$course1,$tilt1 ,$roll1);
   $actuators.=makeModelTilt($lon,$lat,$alt, $course1  ,$tilt1, $roll1, 100,"models/T.dae");
   $actuators.=makeModelTilt($lon,$lat,$alt, $course2  ,$tilt2, $roll2, 100,"models/T.dae");

   $easting-=50;
   ($lat, $lon) = utm_to_latlon($ellipsoid, $z, $easting, $northing);
   ($course1 ,$tilt1 ,$roll1)=(0,90,0);
   ($course2 ,$tilt2 ,$roll2)=convertCoordinateFrame(20,0,0,$course1,$tilt1 ,$roll1);
   $actuators.=makeModelTilt($lon,$lat,$alt, $course1  ,$tilt1, $roll1, 100,"models/T.dae");
   $actuators.=makeModelTilt($lon,$lat,$alt, $course2  ,$tilt2, $roll2, 100,"models/T.dae");

   $easting-=50;
   ($lat, $lon) = utm_to_latlon($ellipsoid, $z, $easting, $northing);
   ($course1 ,$tilt1 ,$roll1)=(0,0,90);
   ($course2 ,$tilt2 ,$roll2)=convertCoordinateFrame(20,0,0,$course1,$tilt1 ,$roll1);
   $actuators.=makeModelTilt($lon,$lat,$alt, $course1  ,$tilt1, $roll1, 100,"models/T.dae");
   $actuators.=makeModelTilt($lon,$lat,$alt, $course2  ,$tilt2, $roll2, 100,"models/T.dae");

   return $actuators;
}
########## I think these are the needed parameters.......
## Params: $f_course,$f_tilt,$f_roll --- the compass course, tilt, and roll (degrees) of the plane
##         $course,$tilt,$roll       --- the angles (degrees) that I wish to show with respect to the a/c
##
## Returns: 
##   $adj_course,$adj_tilt,$adj_roll  
##       -- the $course,$tilt,$roll after being adjusted for the for the cooridinate 
##          frame defined by $f_course,$f_tilt,$f_roll
use Math::Trig qw(deg2rad rad2deg);
sub convertCoordinateFrame {
   print "\n\n";
   print "input =".join(",",map {sprintf"%7.3f",$_} @_)."\n";
   my ($x,$y,$z,$f_x,$f_y,$f_z)=map {deg2rad($_)} @_; ###f_ = frame
   my ($xp,$yp,$zp);

   ############ These are not the correct conversions..............
   $xp=$x*cos($f_y)*cos($f_z)+$y*cos($f_x)*sin($f_z)+$z*cos($f_x)*sin($f_y);
   $yp=$x*cos($f_y)*sin($f_z)+$y*cos($f_x)*cos($f_z)+$z*sin($f_x)*cos($f_y);
   $zp=$x*sin($f_y)*cos($f_z)+$y*cos($f_x)*sin($f_z)+$z*cos($f_x)*cos($f_y);
   #$xp=$f_x;
   #$yp=$f_y;
   #$zp=$f_z;

   #print q"   $f_x,$f_y,$f_z =".sprintf("%6.3f,%6.3f,%6.3f",$f_x ,$f_y ,$f_z )."\n";
   #print q"$x ,$y ,$z =".sprintf("%6.3f,%6.3f,%6.3f",$x ,$y ,$z )."\n";
   #print q"$xp,$yp,$zp=".sprintf("%6.3f,%6.3f,%6.3f",$xp,$yp,$zp)."\n";
   #print q"$xp,$yp,$zp="."$xp,$yp,$zp\n";

   my @ret=map {rad2deg($_)} ($xp,$yp,$zp);
   print "output=".join(",",map {sprintf"%7.3f",$_} @ret)."\n";
   return @ret;
}
#sub convertCoordinateFrame {
#   my ($course,$tilt,$roll,$f_course,$f_tilt,$f_roll)=@_; ###f_ = frame
#   my ($adj_course,$adj_tilt,$adj_roll);
#
#   ############ These are not the correct conversions..............
#   ############ I expect some sin() and cos() functions
#   $adj_course=$course+$f_course;
#   $adj_tilt  =$tilt  +$f_tilt  ;
#   $adj_roll  =sin($roll)  +$f_roll  ;
#
#   return $adj_course,$adj_tilt,$adj_roll;
#}
#####################################################################################################
#####################################################################################################
#####################################################################################################
######################## End Debug: Trying to figure out how to do the rotation!!!!!! ###############
#####################################################################################################
#####################################################################################################
#####################################################################################################


sub getTilt {
   my ($curr,$min,$max)=@_;
   #T28d-Trojan
   my $pct=($curr-$min)/($max-$min);
   my $deg=($pct-0.5)*90;
   #print q"($min,$max,$curr,$pct,$deg)="."($min,$max,$curr,$pct,$deg)\n";
   return ;
}
sub makeBoundingBox {
   my ($ellipsoid, $z_lv,
       $min_easting  , $max_easting  ,
       $min_northing , $max_northing ,
       $min_alt      , $max_alt      ,
   )=@_;
   ####TODO: put marks up the walls at 10meters
   my $cord1=getCord($ellipsoid, $z_lv,$min_easting, $min_northing, $min_alt);
   my $cord2=getCord($ellipsoid, $z_lv,$max_easting, $min_northing, $min_alt);
   my $cord3=getCord($ellipsoid, $z_lv,$min_easting, $max_northing, $min_alt);
   my $cord4=getCord($ellipsoid, $z_lv,$max_easting, $max_northing, $min_alt);
   my $cord5=getCord($ellipsoid, $z_lv,$min_easting, $min_northing, $max_alt);
   my $cord6=getCord($ellipsoid, $z_lv,$max_easting, $min_northing, $max_alt);
   my $cord7=getCord($ellipsoid, $z_lv,$min_easting, $max_northing, $max_alt);
   my $cord8=getCord($ellipsoid, $z_lv,$max_easting, $max_northing, $max_alt);
   my ($walls1,$walls2,$all);
   $walls1.=getPoly("1","bottom","$cord1\n $cord2\n $cord4\n $cord3\n");      
   $walls1.=getPoly("1","north" ,"$cord7\n $cord8\n $cord4\n $cord3\n");      
   $walls1.=getPoly("1","east"  ,"$cord7\n $cord3\n $cord1\n $cord5\n");
   $walls2.=getPoly("0","south" ,"$cord1\n $cord2\n $cord6\n $cord5\n");      
   $walls2.=getPoly("0","west"  ,"$cord2\n $cord6\n $cord8\n $cord4\n");      
   #$walls2.=getPoly("0","top"   ,"$cord5\n $cord6\n $cord8\n $cord7\n");      
   $all.=getFolder("NorthEastBottom",$walls1    ,"1","");
   $all.=getFolder("SouthWest"      ,$walls2    ,"0","");
   return $all;
}
sub getCord{
   my ($ellipsoid, $z_lv, $easting, $northing, $alt)=@_;
   my ($lat, $lon) = utm_to_latlon($ellipsoid, $z_lv, $easting, $northing);
   return "$lon,$lat,$alt\n";
}
sub getPoly {
  my ($vis,$name,$cords)=@_;
  my $style="yellowLine";
  return q"
<Placemark>
    <name>".$name.q"</name>
    <visibility>".$vis.q"</visibility>
    <Polygon>
      <extrude>0</extrude>
      <altitudeMode>absolute</altitudeMode>
      <outerBoundaryIs>
        <LinearRing>
          <coordinates>".$cords.q"</coordinates>
        </LinearRing>
        <styleUrl>".$style.q"</styleUrl>
      </outerBoundaryIs>
    </Polygon>
  </Placemark>
";
}
sub getFolder {
   my ($name,$contents,$vis,$desc)=@_;
   return "    <Folder>
        <name>".$name."</name>
        <description>".$desc."</description>
        <visibility>".$vis."</visibility>
        ".$contents."
            </Folder>
        ";
}
sub makeModelTilt(){
   my ($lon,$lat,$alt,$heading,$tilt,$roll,$scale,$model)=@_;
   my $placeMark="        <Placemark>
            <name>Model</name>
            <description>"."</description>
            <Style id='default'>
            </Style>
            <Model>
                <altitudeMode>absolute</altitudeMode>
                <Location>
                    <longitude>".$lon."</longitude>
                    <latitude>".$lat."</latitude>
                    <altitude>".$alt."</altitude>
                </Location>
                <Orientation>
                    <heading>".$heading."</heading>
                    <tilt>".$tilt."</tilt>
                    <roll>".$roll."</roll>
                </Orientation>
                <Scale>
                    <x>".$scale."</x>
                    <y>".$scale."</y>
                    <z>".$scale."</z>
                </Scale>
                <Link>
                    <href>".$model."</href>
                </Link>
            </Model>
        </Placemark>
     ";
     return $placeMark;
}
sub getStyle {
         my ($ppz_mode)=(@_);
         my $color="blueLine";         
         if(     $ppz_mode eq "0") {$color="greenLine";}
         elsif(  $ppz_mode eq "1") {$color="blueLine";}
         elsif(  $ppz_mode eq "2") {$color="redLine";}
         elsif(  $ppz_mode eq "3") {$color="yellowLine";}
         else {die "invalid PPZ mode ($ppz_mode)";}
         return $color;
}
sub header {
      my($OUT,$name,$desc)=@_;
      print $OUT q\<?xml version="1.0" encoding="UTF-8"?>
<kml xmlns="http://www.opengis.net/kml/2.2">
  <Document>
    <name>\.$name.q\</name>
    <description>\.$desc.q\</description>
    <Style id="yellowLine">
      <LineStyle>
        <color>7fffff00</color>
        <width>7</width>
      </LineStyle>
      <PolyStyle>
        <color>7f00ff00</color>
      </PolyStyle>
    </Style>
    <Style id="redLine">
      <LineStyle>
        <color>7f0000ff</color>
        <width>7</width>
      </LineStyle>
    </Style>
    <Style id="greenLine">
      <LineStyle>
        <color>7f00ff00</color>
        <width>7</width>
      </LineStyle>
    </Style>
    <Style id="blueLine">
      <LineStyle>
        <color>7fff0000</color>
        <width>7</width>
      </LineStyle>
    </Style>
\;
#####################stuff for the plane
      print $OUT q\
	<Style id="sn_airports1">
		<IconStyle>
			<color>ff00ff00</color>
			<scale>1.2</scale>
			<Icon>
				<href>http://maps.google.com/mapfiles/kml/shapes/airports.png</href>
			</Icon>
			<hotSpot x="0.5" y="0" xunits="fraction" yunits="fraction"/>
		</IconStyle>
		<ListStyle>
		</ListStyle>
	</Style>
	<Style id="sh_airports1">
		<IconStyle>
			<scale>1.4</scale>
			<Icon>
				<href>http://maps.google.com/mapfiles/kml/shapes/airports.png</href>
			</Icon>
			<hotSpot x="0.5" y="0" xunits="fraction" yunits="fraction"/>
		</IconStyle>
	</Style>
	<StyleMap id="msn_airports_launch">
		<Pair>
			<key>normal</key>
			<styleUrl>#sn_airports1</styleUrl>
		</Pair>
		<Pair>
			<key>highlight</key>
			<styleUrl>#sh_airports1</styleUrl>
		</Pair>
	</StyleMap>

	<Style id="sn_airports2">
		<IconStyle>
			<color>ff0055ff</color>
			<scale>1.2</scale>
			<Icon>
				<href>http://maps.google.com/mapfiles/kml/shapes/airports.png</href>
			</Icon>
			<hotSpot x="0.5" y="0" xunits="fraction" yunits="fraction"/>
		</IconStyle>
		<ListStyle>
		</ListStyle>
	</Style>
	<Style id="sh_airports2">
		<IconStyle>
			<color>ff0055ff</color>
			<scale>1.4</scale>
			<Icon>
				<href>http://maps.google.com/mapfiles/kml/shapes/airports.png</href>
			</Icon>
			<hotSpot x="0.5" y="0" xunits="fraction" yunits="fraction"/>
		</IconStyle>
		<ListStyle>
		</ListStyle>
	</Style>
	<StyleMap id="msn_airports_land">
		<Pair>
			<key>normal</key>
			<styleUrl>#sn_airports2</styleUrl>
		</Pair>
		<Pair>
			<key>highlight</key>
			<styleUrl>#sh_airports2</styleUrl>
		</Pair>
	</StyleMap>
\;
}
sub placeMarkPlane {
##http://maps.google.com/mapfiles/kml/shapes/airports.png   Can this be shorter?
      my($name,$style,$lat,$long,$alt,$vis,$tilts)=@_;
      return q" 
	<Placemark>
		<name>".$name.q"</name>
                <visibility>".$vis."</visibility>
		<styleUrl>#".$style.q"</styleUrl>
		<Point>
			<coordinates>"."$long,$lat,$alt".q"</coordinates>
                        <altitudeMode>absolute</altitudeMode>
		</Point>
	</Placemark>
";
}
sub makeLeg{
      my($leg,$mode,$style,$cords,$nav_cords,$tilts)=@_;
      my $tiltsFolder=getFolder($leg." Tilts",$tilts,"1","");#$name,$contents,$vis,$desc
      my $tmp= $tiltsFolder.q"    
      <Placemark>
         <name>".$leg.q"</name>
         <description>"."$mode".q"</description>
         <styleUrl>#".$style.q"</styleUrl>
         <LineString>
           <extrude>0</extrude>
           <tessellate>0</tessellate>
           <altitudeMode>absolute</altitudeMode>
           <coordinates>".$cords.q"
           </coordinates>
         </LineString>
      </Placemark>
         <Placemark>
            <name>nav_".$leg.q"</name>
            <description>"."$mode".q"</description>
            <styleUrl>#".$style.q"</styleUrl>
            <LineString>
              <extrude>0</extrude>
              <tessellate>0</tessellate>
              <altitudeMode>absolute</altitudeMode>
              <coordinates>".$nav_cords.q"
              </coordinates>
            </LineString>
         </Placemark>
      ";
      return getFolder($leg,$tmp,"1","");
}
sub footer{
      my($OUT)=@_;
      print $OUT q\  </Document>
</kml>
\;
}


sub writeModel(){
      my($dest)=@_;
      my ($base,$path,$type) = fileparse($dest);
      my $dest = File::Spec->catfile($path,"models");
      mkdir "$dest";

      $dest = File::Spec->catfile($path,"models","SimplePlane.dae");
      unless(open PLANE,">$dest"){ print "cannot open file '$dest'.\n";return;}

      my $dst2 = File::Spec->catfile($path,"models","T.dae");
      unless(open T,">$dst2"){ print "cannot open file '$dst2'.\n";return;}

      print PLANE q\<?xml version="1.0" encoding="utf-8"?>
<COLLADA xmlns="http://www.collada.org/2005/11/COLLADASchema" version="1.4.1">
   <asset>
      <contributor>
         <authoring_tool>Google SketchUp 7.0.10247</authoring_tool>
      </contributor>
      <created>2009-12-08T13:49:15Z</created>
      <modified>2009-12-08T13:49:15Z</modified>
      <unit name="inches" meter="0.0254"/>
      <up_axis>Z_UP</up_axis>
   </asset>
   <library_materials>
      <material id="material_0_16ID" name="material_0_16">
         <instance_effect url="#material_0_16-effect"/>
      </material>
      <material id="material_1_0ID" name="material_1_0">
         <instance_effect url="#material_1_0-effect"/>
      </material>
      <material id="material_2_16ID" name="material_2_16">
         <instance_effect url="#material_2_16-effect"/>
      </material>
      <material id="material_3_16ID" name="material_3_16">
         <instance_effect url="#material_3_16-effect"/>
      </material>
      <material id="material_4_16ID" name="material_4_16">
         <instance_effect url="#material_4_16-effect"/>
      </material>
   </library_materials>
   <library_effects>
      <effect id="material_0_16-effect" name="material_0_16-effect">
         <profile_COMMON>
            <technique sid="COMMON">
               <lambert>
                  <emission>
                     <color>0.000000 0.000000 0.000000 1</color>
                  </emission>
                  <ambient>
                     <color>0.000000 0.000000 0.000000 1</color>
                  </ambient>
                  <diffuse>
                     <color>0.600000 0.600000 1.000000 1</color>
                  </diffuse>
                  <transparent>
                     <color>1 1 1 1</color>
                  </transparent>
                  <transparency>
                     <float>0.000000</float>
                  </transparency>
               </lambert>
            </technique>
            <extra>
               <technique profile="GOOGLEEARTH">
                  <double_sided>1</double_sided>
               </technique>
            </extra>
         </profile_COMMON>
      </effect>
      <effect id="material_1_0-effect" name="material_1_0-effect">
         <profile_COMMON>
            <technique sid="COMMON">
               <lambert>
                  <emission>
                     <color>0.000000 0.000000 0.000000 1</color>
                  </emission>
                  <ambient>
                     <color>0.000000 0.000000 0.000000 1</color>
                  </ambient>
                  <diffuse>
                     <color>0.000000 0.000000 0.000000 1</color>
                  </diffuse>
                  <transparent>
                     <color>1 1 1 1</color>
                  </transparent>
                  <transparency>
                     <float>0.000000</float>
                  </transparency>
               </lambert>
            </technique>
            <extra>
               <technique profile="GOOGLEEARTH">
                  <double_sided>1</double_sided>
               </technique>
            </extra>
         </profile_COMMON>
      </effect>
      <effect id="material_2_16-effect" name="material_2_16-effect">
         <profile_COMMON>
            <technique sid="COMMON">
               <lambert>
                  <emission>
                     <color>0.000000 0.000000 0.000000 1</color>
                  </emission>
                  <ambient>
                     <color>0.000000 0.000000 0.000000 1</color>
                  </ambient>
                  <diffuse>
                     <color>0.200000 0.000000 0.200000 1</color>
                  </diffuse>
                  <transparent>
                     <color>1 1 1 1</color>
                  </transparent>
                  <transparency>
                     <float>0.000000</float>
                  </transparency>
               </lambert>
            </technique>
            <extra>
               <technique profile="GOOGLEEARTH">
                  <double_sided>1</double_sided>
               </technique>
            </extra>
         </profile_COMMON>
      </effect>
      <effect id="material_3_16-effect" name="material_3_16-effect">
         <profile_COMMON>
            <technique sid="COMMON">
               <lambert>
                  <emission>
                     <color>0.000000 0.000000 0.000000 1</color>
                  </emission>
                  <ambient>
                     <color>0.000000 0.000000 0.000000 1</color>
                  </ambient>
                  <diffuse>
                     <color>0.000000 0.600000 0.000000 1</color>
                  </diffuse>
                  <transparent>
                     <color>1 1 1 1</color>
                  </transparent>
                  <transparency>
                     <float>0.000000</float>
                  </transparency>
               </lambert>
            </technique>
            <extra>
               <technique profile="GOOGLEEARTH">
                  <double_sided>1</double_sided>
               </technique>
            </extra>
         </profile_COMMON>
      </effect>
      <effect id="material_4_16-effect" name="material_4_16-effect">
         <profile_COMMON>
            <technique sid="COMMON">
               <lambert>
                  <emission>
                     <color>0.000000 0.000000 0.000000 1</color>
                  </emission>
                  <ambient>
                     <color>0.000000 0.000000 0.000000 1</color>
                  </ambient>
                  <diffuse>
                     <color>1.000000 0.247059 0.000000 1</color>
                  </diffuse>
                  <transparent>
                     <color>1 1 1 1</color>
                  </transparent>
                  <transparency>
                     <float>0.000000</float>
                  </transparency>
               </lambert>
            </technique>
            <extra>
               <technique profile="GOOGLEEARTH">
                  <double_sided>1</double_sided>
               </technique>
            </extra>
         </profile_COMMON>
      </effect>
   </library_effects>
   <library_geometries>
      <geometry id="mesh1-geometry" name="mesh1-geometry">
         <mesh>
            <source id="mesh1-geometry-position">
               <float_array id="mesh1-geometry-position-array" count="156">35.000000 12.414358 6.000000 7.000000 12.414358 5.000000 35.000000 12.414358 5.000000 7.000000 12.414358 6.000000 42.000000 21.414358 5.000000 35.982566 12.708823 6.000000 42.000000 21.414358 6.000000 6.017434 12.708823 5.000000 35.982566 12.708823 5.000000 6.017434 12.708823 6.000000 5.074123 13.111703 5.000000 36.925877 13.111703 5.000000 36.925877 13.111703 6.000000 5.074123 13.111703 6.000000 4.181989 13.617908 5.000000 37.818011 13.617908 5.000000 37.818011 13.617908 6.000000 4.181989 13.617908 6.000000 3.352304 14.221040 5.000000 38.647696 14.221040 5.000000 38.647696 14.221040 6.000000 3.352304 14.221040 6.000000 2.595552 14.913480 5.000000 39.404448 14.913480 5.000000 39.404448 14.913480 6.000000 2.595552 14.913480 6.000000 1.921296 15.686477 5.000000 40.078704 15.686477 5.000000 40.078704 15.686477 6.000000 1.921296 15.686477 6.000000 1.338054 16.530263 5.000000 40.661946 16.530263 5.000000 40.661946 16.530263 6.000000 1.338054 16.530263 6.000000 0.853198 17.434178 5.000000 41.146802 17.434178 5.000000 41.146802 17.434178 6.000000 0.853198 17.434178 6.000000 0.472853 18.386797 5.000000 41.527147 18.386797 5.000000 41.527147 18.386797 6.000000 0.472853 18.386797 6.000000 0.201826 19.376085 5.000000 41.798174 19.376085 5.000000 41.798174 19.376085 6.000000 0.201826 19.376085 6.000000 0.043542 20.389541 5.000000 41.956458 20.389541 5.000000 41.956458 20.389541 6.000000 0.043542 20.389541 6.000000 0.000000 21.414358 5.000000 0.000000 21.414358 6.000000 </float_array>
               <technique_common>
                  <accessor source="#mesh1-geometry-position-array" count="52" stride="3">
                     <param name="X" type="float"/>
                     <param name="Y" type="float"/>
                     <param name="Z" type="float"/>
                  </accessor>
               </technique_common>
            </source>
            <source id="mesh1-geometry-normal">
               <float_array id="mesh1-geometry-normal-array" count="90">-0.000000 -1.000000 0.000000 0.000000 0.000000 -1.000000 0.287075 -0.957908 0.000000 0.340460 -0.940259 0.000000 0.000000 0.000000 1.000000 -0.287075 -0.957908 0.000000 -0.340460 -0.940259 0.000000 0.443837 -0.896108 0.000000 -0.443837 -0.896108 0.000000 -0.541605 -0.840633 0.000000 0.541605 -0.840633 0.000000 -0.632529 -0.774536 0.000000 0.632529 -0.774536 0.000000 -0.715461 -0.698653 0.000000 0.715461 -0.698653 0.000000 -0.789352 -0.613941 0.000000 0.789352 -0.613941 0.000000 -0.853269 -0.521471 0.000000 0.853269 -0.521471 0.000000 -0.906404 -0.422412 0.000000 0.906404 -0.422412 0.000000 -0.948086 -0.318015 0.000000 0.948086 -0.318015 0.000000 -0.977787 -0.209600 0.000000 0.977787 -0.209600 0.000000 -0.995133 -0.098536 0.000000 0.999099 -0.042449 0.000000 0.995133 -0.098536 0.000000 -0.999099 -0.042449 0.000000 0.000000 1.000000 0.000000 </float_array>
               <technique_common>
                  <accessor source="#mesh1-geometry-normal-array" count="30" stride="3">
                     <param name="X" type="float"/>
                     <param name="Y" type="float"/>
                     <param name="Z" type="float"/>
                  </accessor>
               </technique_common>
            </source>
            <vertices id="mesh1-geometry-vertex">
               <input semantic="POSITION" source="#mesh1-geometry-position"/>
            </vertices>
            <triangles material="material_0_16" count="100">
               <input semantic="VERTEX" source="#mesh1-geometry-vertex" offset="0"/>
               <input semantic="NORMAL" source="#mesh1-geometry-normal" offset="1"/>
               <p>0 0 1 0 2 0 1 0 0 0 3 0 1 1 4 1 2 1 2 2 5 3 0 2 6 4 3 4 0 4 3 5 7 6 1 5 7 1 4 1 1 1 2 1 4 1 8 1 5 3 2 2 8 3 6 4 0 4 5 4 6 4 9 4 3 4 7 6 3 5 9 6 10 1 4 1 7 1 8 1 4 1 11 1 8 3 12 7 5 3 6 4 5 4 12 4 6 4 13 4 9 4 9 6 10 8 7 6 14 1 4 1 10 1 11 1 4 1 15 1 12 7 8 3 11 7 6 4 12 4 16 4 6 4 17 4 13 4 10 8 9 6 13 8 18 1 4 1 14 1 13 8 14 9 10 8 15 1 4 1 19 1 16 10 11 7 15 10 11 7 16 10 12 7 6 4 16 4 20 4 6 4 21 4 17 4 14 9 13 8 17 9 22 1 4 1 18 1 17 9 18 11 14 9 19 1 4 1 23 1 20 12 15 10 19 12 15 10 20 12 16 10 6 4 20 4 24 4 6 4 25 4 21 4 18 11 17 9 21 11 26 1 4 1 22 1 21 11 22 13 18 11 23 1 4 1 27 1 24 14 19 12 23 14 19 12 24 14 20 12 6 4 24 4 28 4 6 4 29 4 25 4 22 13 21 11 25 13 30 1 4 1 26 1 22 13 29 15 26 15 27 1 4 1 31 1 27 16 24 14 23 14 24 14 27 16 28 16 6 4 28 4 32 4 6 4 33 4 29 4 29 15 22 13 25 13 34 1 4 1 30 1 26 15 33 17 30 17 33 17 26 15 29 15 31 1 4 1 35 1 31 18 28 16 27 16 28 16 31 18 32 18 6 4 32 4 36 4 6 4 37 4 33 4 38 1 4 1 34 1 30 17 37 19 34 19 37 19 30 17 33 17 35 1 4 1 39 1 35 20 32 18 31 18 32 18 35 20 36 20 6 4 36 4 40 4 6 4 41 4 37 4 42 1 4 1 38 1 34 19 41 21 38 21 41 21 34 19 37 19 39 1 4 1 43 1 39 22 36 20 35 20 36 20 39 22 40 22 6 4 40 4 44 4 6 4 45 4 41 4 46 1 4 1 42 1 38 21 45 23 42 23 45 23 38 21 41 21 43 1 4 1 47 1 43 24 40 22 39 22 40 22 43 24 44 24 6 4 44 4 48 4 6 4 49 4 45 4 4 1 46 1 50 1 42 23 49 25 46 25 49 25 42 23 45 23 4 26 48 27 47 27 47 27 44 24 43 24 44 24 47 27 48 27 48 27 4 26 6 26 49 4 6 4 51 4 46 25 51 28 50 28 51 29 4 29 50 29 51 28 46 25 49 25 4 29 51 29 6 29 </p>
            </triangles>
         </mesh>
      </geometry>
      <geometry id="mesh2-geometry" name="mesh2-geometry">
         <mesh>
            <source id="mesh2-geometry-position">
               <float_array id="mesh2-geometry-position-array" count="156">0.000000 21.414358 6.000000 42.000000 21.414358 6.000000 0.043542 20.389541 6.000000 0.000000 21.414358 5.000000 42.000000 21.414358 5.000000 41.956458 20.389541 6.000000 0.201826 19.376085 6.000000 0.043542 20.389541 5.000000 41.956458 20.389541 5.000000 41.798174 19.376085 6.000000 0.472853 18.386797 6.000000 0.201826 19.376085 5.000000 41.798174 19.376085 5.000000 41.527147 18.386797 6.000000 0.853198 17.434178 6.000000 0.472853 18.386797 5.000000 41.527147 18.386797 5.000000 41.146802 17.434178 6.000000 1.338054 16.530263 6.000000 0.853198 17.434178 5.000000 41.146802 17.434178 5.000000 40.661946 16.530263 6.000000 1.921296 15.686477 6.000000 1.338054 16.530263 5.000000 40.661946 16.530263 5.000000 40.078704 15.686477 6.000000 2.595552 14.913480 6.000000 1.921296 15.686477 5.000000 40.078704 15.686477 5.000000 39.404448 14.913480 6.000000 3.352304 14.221040 6.000000 2.595552 14.913480 5.000000 39.404448 14.913480 5.000000 38.647696 14.221040 6.000000 4.181989 13.617908 6.000000 3.352304 14.221040 5.000000 38.647696 14.221040 5.000000 37.818011 13.617908 6.000000 5.074123 13.111703 6.000000 4.181989 13.617908 5.000000 37.818011 13.617908 5.000000 36.925877 13.111703 6.000000 6.017434 12.708823 6.000000 5.074123 13.111703 5.000000 36.925877 13.111703 5.000000 35.982566 12.708823 6.000000 7.000000 12.414358 6.000000 6.017434 12.708823 5.000000 35.982566 12.708823 5.000000 35.000000 12.414358 6.000000 7.000000 12.414358 5.000000 35.000000 12.414358 5.000000 </float_array>
               <technique_common>
                  <accessor source="#mesh2-geometry-position-array" count="52" stride="3">
                     <param name="X" type="float"/>
                     <param name="Y" type="float"/>
                     <param name="Z" type="float"/>
                  </accessor>
               </technique_common>
            </source>
            <vertices id="mesh2-geometry-vertex">
               <input semantic="POSITION" source="#mesh2-geometry-position"/>
            </vertices>
            <lines material="material_1_0" count="56">
               <input semantic="VERTEX" source="#mesh2-geometry-vertex" offset="0"/>
               <p>0 1 2 0 0 3 4 1 1 5 6 2 3 4 7 3 4 8 5 9 10 6 11 7 8 12 9 13 14 10 15 11 12 16 13 17 18 14 19 15 16 20 17 21 22 18 23 19 20 24 21 25 26 22 27 23 24 28 25 29 30 26 31 27 28 32 29 33 34 30 35 31 32 36 33 37 38 34 39 35 36 40 37 41 42 38 43 39 40 44 41 45 46 42 47 43 44 48 45 49 46 50 49 46 50 47 48 51 49 51 51 50 </p>
            </lines>
         </mesh>
      </geometry>
      <geometry id="mesh3-geometry" name="mesh3-geometry">
         <mesh>
            <source id="mesh3-geometry-position">
               <float_array id="mesh3-geometry-position-array" count="24">22.587387 55.000000 0.000000 19.587387 0.000000 0.000000 19.587387 55.000000 0.000000 22.587387 0.000000 0.000000 19.587387 55.000000 5.000000 22.587387 0.000000 5.000000 19.587387 0.000000 5.000000 22.587387 55.000000 5.000000 </float_array>
               <technique_common>
                  <accessor source="#mesh3-geometry-position-array" count="8" stride="3">
                     <param name="X" type="float"/>
                     <param name="Y" type="float"/>
                     <param name="Z" type="float"/>
                  </accessor>
               </technique_common>
            </source>
            <source id="mesh3-geometry-normal">
               <float_array id="mesh3-geometry-normal-array" count="18">0.000000 0.000000 -1.000000 -1.000000 0.000000 0.000000 0.000000 1.000000 0.000000 1.000000 0.000000 0.000000 0.000000 -1.000000 0.000000 0.000000 0.000000 1.000000 </float_array>
               <technique_common>
                  <accessor source="#mesh3-geometry-normal-array" count="6" stride="3">
                     <param name="X" type="float"/>
                     <param name="Y" type="float"/>
                     <param name="Z" type="float"/>
                  </accessor>
               </technique_common>
            </source>
            <vertices id="mesh3-geometry-vertex">
               <input semantic="POSITION" source="#mesh3-geometry-position"/>
            </vertices>
            <triangles material="material_2_16" count="12">
               <input semantic="VERTEX" source="#mesh3-geometry-vertex" offset="0"/>
               <input semantic="NORMAL" source="#mesh3-geometry-normal" offset="1"/>
               <p>0 0 1 0 2 0 1 0 0 0 3 0 1 1 4 1 2 1 4 2 0 2 2 2 0 3 5 3 3 3 5 4 1 4 3 4 4 1 1 1 6 1 0 2 4 2 7 2 5 3 0 3 7 3 1 4 5 4 6 4 5 5 4 5 6 5 4 5 5 5 7 5 </p>
            </triangles>
         </mesh>
      </geometry>
      <geometry id="mesh4-geometry" name="mesh4-geometry">
         <mesh>
            <source id="mesh4-geometry-position">
               <float_array id="mesh4-geometry-position-array" count="24">22.587387 55.000000 0.000000 19.587387 55.000000 0.000000 22.587387 0.000000 0.000000 22.587387 55.000000 5.000000 19.587387 0.000000 0.000000 19.587387 55.000000 5.000000 22.587387 0.000000 5.000000 19.587387 0.000000 5.000000 </float_array>
               <technique_common>
                  <accessor source="#mesh4-geometry-position-array" count="8" stride="3">
                     <param name="X" type="float"/>
                     <param name="Y" type="float"/>
                     <param name="Z" type="float"/>
                  </accessor>
               </technique_common>
            </source>
            <vertices id="mesh4-geometry-vertex">
               <input semantic="POSITION" source="#mesh4-geometry-position"/>
            </vertices>
            <lines material="material_1_0" count="12">
               <input semantic="VERTEX" source="#mesh4-geometry-vertex" offset="0"/>
               <p>0 1 0 2 3 0 1 4 1 5 2 4 6 2 3 6 5 3 4 7 5 7 7 6 </p>
            </lines>
         </mesh>
      </geometry>
      <geometry id="mesh5-geometry" name="mesh5-geometry">
         <mesh>
            <source id="mesh5-geometry-position">
               <float_array id="mesh5-geometry-position-array" count="24">12.584334 55.269067 6.000000 12.584334 48.769067 5.000000 12.584334 48.769067 6.000000 12.584334 55.269067 5.000000 29.584334 48.769067 6.000000 29.584334 55.269067 5.000000 29.584334 48.769067 5.000000 29.584334 55.269067 6.000000 </float_array>
               <technique_common>
                  <accessor source="#mesh5-geometry-position-array" count="8" stride="3">
                     <param name="X" type="float"/>
                     <param name="Y" type="float"/>
                     <param name="Z" type="float"/>
                  </accessor>
               </technique_common>
            </source>
            <source id="mesh5-geometry-normal">
               <float_array id="mesh5-geometry-normal-array" count="18">-1.000000 0.000000 0.000000 0.000000 -1.000000 0.000000 0.000000 0.000000 1.000000 0.000000 1.000000 0.000000 0.000000 0.000000 -1.000000 1.000000 0.000000 0.000000 </float_array>
               <technique_common>
                  <accessor source="#mesh5-geometry-normal-array" count="6" stride="3">
                     <param name="X" type="float"/>
                     <param name="Y" type="float"/>
                     <param name="Z" type="float"/>
                  </accessor>
               </technique_common>
            </source>
            <vertices id="mesh5-geometry-vertex">
               <input semantic="POSITION" source="#mesh5-geometry-position"/>
            </vertices>
            <triangles material="material_3_16" count="12">
               <input semantic="VERTEX" source="#mesh5-geometry-vertex" offset="0"/>
               <input semantic="NORMAL" source="#mesh5-geometry-normal" offset="1"/>
               <p>0 0 1 0 2 0 1 0 0 0 3 0 1 1 4 1 2 1 4 2 0 2 2 2 0 3 5 3 3 3 5 4 1 4 3 4 4 1 1 1 6 1 0 2 4 2 7 2 5 3 0 3 7 3 1 4 5 4 6 4 5 5 4 5 6 5 4 5 5 5 7 5 </p>
            </triangles>
         </mesh>
      </geometry>
      <geometry id="mesh6-geometry" name="mesh6-geometry">
         <mesh>
            <source id="mesh6-geometry-position">
               <float_array id="mesh6-geometry-position-array" count="24">29.584334 55.269067 5.000000 29.584334 48.769067 5.000000 29.584334 55.269067 6.000000 12.584334 55.269067 5.000000 29.584334 48.769067 6.000000 12.584334 48.769067 5.000000 12.584334 55.269067 6.000000 12.584334 48.769067 6.000000 </float_array>
               <technique_common>
                  <accessor source="#mesh6-geometry-position-array" count="8" stride="3">
                     <param name="X" type="float"/>
                     <param name="Y" type="float"/>
                     <param name="Z" type="float"/>
                  </accessor>
               </technique_common>
            </source>
            <vertices id="mesh6-geometry-vertex">
               <input semantic="POSITION" source="#mesh6-geometry-position"/>
            </vertices>
            <lines material="material_1_0" count="12">
               <input semantic="VERTEX" source="#mesh6-geometry-vertex" offset="0"/>
               <p>0 1 0 2 3 0 1 4 1 5 2 4 6 2 5 3 3 6 4 7 5 7 7 6 </p>
            </lines>
         </mesh>
      </geometry>
      <geometry id="mesh7-geometry" name="mesh7-geometry">
         <mesh>
            <source id="mesh7-geometry-position">
               <float_array id="mesh7-geometry-position-array" count="24">21.084334 55.019067 13.000000 21.484334 55.019067 5.000000 21.084334 55.019067 5.000000 21.484334 55.019067 13.000000 21.084334 49.019067 5.000000 21.484334 49.019067 13.000000 21.484334 49.019067 5.000000 21.084334 49.019067 13.000000 </float_array>
               <technique_common>
                  <accessor source="#mesh7-geometry-position-array" count="8" stride="3">
                     <param name="X" type="float"/>
                     <param name="Y" type="float"/>
                     <param name="Z" type="float"/>
                  </accessor>
               </technique_common>
            </source>
            <source id="mesh7-geometry-normal">
               <float_array id="mesh7-geometry-normal-array" count="18">0.000000 1.000000 0.000000 0.000000 0.000000 -1.000000 -1.000000 0.000000 0.000000 0.000000 0.000000 1.000000 1.000000 0.000000 0.000000 0.000000 -1.000000 0.000000 </float_array>
               <technique_common>
                  <accessor source="#mesh7-geometry-normal-array" count="6" stride="3">
                     <param name="X" type="float"/>
                     <param name="Y" type="float"/>
                     <param name="Z" type="float"/>
                  </accessor>
               </technique_common>
            </source>
            <vertices id="mesh7-geometry-vertex">
               <input semantic="POSITION" source="#mesh7-geometry-position"/>
            </vertices>
            <triangles material="material_4_16" count="12">
               <input semantic="VERTEX" source="#mesh7-geometry-vertex" offset="0"/>
               <input semantic="NORMAL" source="#mesh7-geometry-normal" offset="1"/>
               <p>0 0 1 0 2 0 1 0 0 0 3 0 1 1 4 1 2 1 4 2 0 2 2 2 0 3 5 3 3 3 5 4 1 4 3 4 4 1 1 1 6 1 0 2 4 2 7 2 5 3 0 3 7 3 1 4 5 4 6 4 5 5 4 5 6 5 4 5 5 5 7 5 </p>
            </triangles>
         </mesh>
      </geometry>
      <geometry id="mesh8-geometry" name="mesh8-geometry">
         <mesh>
            <source id="mesh8-geometry-position">
               <float_array id="mesh8-geometry-position-array" count="24">21.484334 55.019067 13.000000 21.484334 55.019067 5.000000 21.084334 55.019067 13.000000 21.484334 49.019067 13.000000 21.084334 55.019067 5.000000 21.484334 49.019067 5.000000 21.084334 49.019067 13.000000 21.084334 49.019067 5.000000 </float_array>
               <technique_common>
                  <accessor source="#mesh8-geometry-position-array" count="8" stride="3">
                     <param name="X" type="float"/>
                     <param name="Y" type="float"/>
                     <param name="Z" type="float"/>
                  </accessor>
               </technique_common>
            </source>
            <vertices id="mesh8-geometry-vertex">
               <input semantic="POSITION" source="#mesh8-geometry-position"/>
            </vertices>
            <lines material="material_1_0" count="12">
               <input semantic="VERTEX" source="#mesh8-geometry-vertex" offset="0"/>
               <p>0 1 0 2 3 0 1 4 1 5 2 4 6 2 5 3 3 6 4 7 5 7 7 6 </p>
            </lines>
         </mesh>
      </geometry>
   </library_geometries>
   <library_nodes>
      <node id="Plane" name="Plane">
         <node id="WING1" name="WING1">
            <node id="mesh1" name="mesh1">
               <instance_geometry url="#mesh1-geometry">
                  <bind_material>
                     <technique_common>
                        <instance_material symbol="material_0_16" target="#material_0_16ID"/>
                     </technique_common>
                  </bind_material>
               </instance_geometry>
            </node>
            <node id="mesh2" name="mesh2">
               <instance_geometry url="#mesh2-geometry">
                  <bind_material>
                     <technique_common>
                        <instance_material symbol="material_1_0" target="#material_1_0ID"/>
                     </technique_common>
                  </bind_material>
               </instance_geometry>
            </node>
         </node>
         <node id="BODY1" name="BODY1">
            <node id="mesh3" name="mesh3">
               <instance_geometry url="#mesh3-geometry">
                  <bind_material>
                     <technique_common>
                        <instance_material symbol="material_2_16" target="#material_2_16ID"/>
                     </technique_common>
                  </bind_material>
               </instance_geometry>
            </node>
            <node id="mesh4" name="mesh4">
               <instance_geometry url="#mesh4-geometry">
                  <bind_material>
                     <technique_common>
                        <instance_material symbol="material_1_0" target="#material_1_0ID"/>
                     </technique_common>
                  </bind_material>
               </instance_geometry>
            </node>
         </node>
         <node id="stablizer1" name="stablizer1">
            <node id="mesh5" name="mesh5">
               <instance_geometry url="#mesh5-geometry">
                  <bind_material>
                     <technique_common>
                        <instance_material symbol="material_3_16" target="#material_3_16ID"/>
                     </technique_common>
                  </bind_material>
               </instance_geometry>
            </node>
            <node id="mesh6" name="mesh6">
               <instance_geometry url="#mesh6-geometry">
                  <bind_material>
                     <technique_common>
                        <instance_material symbol="material_1_0" target="#material_1_0ID"/>
                     </technique_common>
                  </bind_material>
               </instance_geometry>
            </node>
         </node>
         <node id="Rudder1" name="Rudder1">
            <node id="mesh7" name="mesh7">
               <instance_geometry url="#mesh7-geometry">
                  <bind_material>
                     <technique_common>
                        <instance_material symbol="material_4_16" target="#material_4_16ID"/>
                     </technique_common>
                  </bind_material>
               </instance_geometry>
            </node>
            <node id="mesh8" name="mesh8">
               <instance_geometry url="#mesh8-geometry">
                  <bind_material>
                     <technique_common>
                        <instance_material symbol="material_1_0" target="#material_1_0ID"/>
                     </technique_common>
                  </bind_material>
               </instance_geometry>
            </node>
         </node>
      </node>
   </library_nodes>
   <library_cameras>
      <camera id="Camera-camera" name="Camera-camera">
         <optics>
            <technique_common>
               <perspective>
                  <xfov>46.666667</xfov>
                  <yfov>35.000000</yfov>
                  <znear>1.000000</znear>
                  <zfar>1000.000000</zfar>
               </perspective>
            </technique_common>
         </optics>
      </camera>
   </library_cameras>
   <library_visual_scenes>
      <visual_scene id="SketchUpScene" name="SketchUpScene">
         <node id="Model" name="Model">
            <node id="Plane1" name="Plane1">
               <matrix>
                  -1.000000 -0.000000 0.000000 21.062500
                  0.000000 -1.000000 0.000000 21.437500
                  0.000000 0.000000 1.000000 -5.000000
                  0.000000 0.000000 0.000000 1.000000
               </matrix>
               <instance_node url="#Plane"/>
            </node>
         </node>
         <node id="Camera" name="Camera">
            <matrix>
               0.852531 0.465192 -0.238302 -26.620765
               -0.522677 0.758767 -0.388691 -53.023602
               0.000000 0.455926 0.890018 93.452456
               0.000000 0.000000 0.000000 1.000000
            </matrix>
            <instance_camera url="#Camera-camera"/>
         </node>
      </visual_scene>
   </library_visual_scenes>
   <scene>
      <instance_visual_scene url="#SketchUpScene"/>
   </scene>
</COLLADA>
\;
#########################################################################################################
#########################################################################################################
#########################################################################################################
#########################################################################################################
#########################################################################################################
#########################################################################################################
#########################################################################################################
#########################################################################################################
#########################################################################################################
#########################################################################################################
#########################################################################################################
#########################################################################################################
#########################################################################################################
#########################################################################################################
print T q\<?xml version="1.0" encoding="utf-8"?>
<COLLADA xmlns="http://www.collada.org/2005/11/COLLADASchema" version="1.4.1">
   <asset>
      <contributor>
         <authoring_tool>Google SketchUp 7.0.10247</authoring_tool>
      </contributor>
      <created>2009-12-08T14:09:10Z</created>
      <modified>2009-12-08T14:09:10Z</modified>
      <unit name="inches" meter="0.0254"/>
      <up_axis>Z_UP</up_axis>
   </asset>
   <library_materials>
      <material id="material_0_16ID" name="material_0_16">
         <instance_effect url="#material_0_16-effect"/>
      </material>
      <material id="material_0_0ID" name="material_0_0">
         <instance_effect url="#material_0_0-effect"/>
      </material>
   </library_materials>
   <library_effects>
      <effect id="material_0_16-effect" name="material_0_16-effect">
         <profile_COMMON>
            <technique sid="COMMON">
               <lambert>
                  <emission>
                     <color>0.000000 0.000000 0.000000 1</color>
                  </emission>
                  <ambient>
                     <color>0.000000 0.000000 0.000000 1</color>
                  </ambient>
                  <diffuse>
                     <color>1.000000 1.000000 1.000000 1</color>
                  </diffuse>
                  <transparent>
                     <color>1 1 1 1</color>
                  </transparent>
                  <transparency>
                     <float>0.000000</float>
                  </transparency>
               </lambert>
            </technique>
            <extra>
               <technique profile="GOOGLEEARTH">
                  <double_sided>1</double_sided>
               </technique>
            </extra>
         </profile_COMMON>
      </effect>
      <effect id="material_0_0-effect" name="material_0_0-effect">
         <profile_COMMON>
            <technique sid="COMMON">
               <lambert>
                  <emission>
                     <color>0.000000 0.000000 0.000000 1</color>
                  </emission>
                  <ambient>
                     <color>0.000000 0.000000 0.000000 1</color>
                  </ambient>
                  <diffuse>
                     <color>0.000000 0.000000 0.000000 1</color>
                  </diffuse>
                  <transparent>
                     <color>1 1 1 1</color>
                  </transparent>
                  <transparency>
                     <float>0.000000</float>
                  </transparency>
               </lambert>
            </technique>
            <extra>
               <technique profile="GOOGLEEARTH">
                  <double_sided>1</double_sided>
               </technique>
            </extra>
         </profile_COMMON>
      </effect>
   </library_effects>
   <library_geometries>
      <geometry id="mesh1-geometry" name="mesh1-geometry">
         <mesh>
            <source id="mesh1-geometry-position">
               <float_array id="mesh1-geometry-position-array" count="48">0.510735 0.023142 7.000000 -0.489265 0.023142 2.000000 0.510735 0.023142 2.000000 -0.489265 0.023142 7.000000 -6.937500 0.023142 2.000000 0.510735 5.023142 2.000000 -0.489265 5.023142 7.000000 7.062500 0.023142 2.000000 -6.937500 5.023142 2.000000 0.510735 5.023142 7.000000 -0.489265 5.023142 2.000000 -6.937500 0.023142 1.000000 7.062500 5.023142 2.000000 7.062500 0.023142 1.000000 -6.937500 5.023142 1.000000 7.062500 5.023142 1.000000 </float_array>
               <technique_common>
                  <accessor source="#mesh1-geometry-position-array" count="16" stride="3">
                     <param name="X" type="float"/>
                     <param name="Y" type="float"/>
                     <param name="Z" type="float"/>
                  </accessor>
               </technique_common>
            </source>
            <source id="mesh1-geometry-normal">
               <float_array id="mesh1-geometry-normal-array" count="18">0.000000 -1.000000 0.000000 1.000000 0.000000 0.000000 0.000000 0.000000 1.000000 -1.000000 0.000000 0.000000 0.000000 1.000000 0.000000 0.000000 0.000000 -1.000000 </float_array>
               <technique_common>
                  <accessor source="#mesh1-geometry-normal-array" count="6" stride="3">
                     <param name="X" type="float"/>
                     <param name="Y" type="float"/>
                     <param name="Z" type="float"/>
                  </accessor>
               </technique_common>
            </source>
            <vertices id="mesh1-geometry-vertex">
               <input semantic="POSITION" source="#mesh1-geometry-position"/>
            </vertices>
            <triangles material="material_0_16" count="28">
               <input semantic="VERTEX" source="#mesh1-geometry-vertex" offset="0"/>
               <input semantic="NORMAL" source="#mesh1-geometry-normal" offset="1"/>
               <p>0 0 1 0 2 0 1 0 0 0 3 0 4 0 2 0 1 0 5 1 0 1 2 1 0 2 6 2 3 2 6 3 1 3 3 3 4 0 7 0 2 0 1 2 8 2 4 2 0 1 5 1 9 1 7 2 5 2 2 2 6 2 0 2 9 2 1 3 6 3 10 3 11 0 7 0 4 0 8 2 1 2 10 2 8 3 11 3 4 3 5 4 6 4 9 4 5 2 7 2 12 2 6 4 5 4 10 4 7 0 11 0 13 0 12 4 8 4 10 4 11 3 8 3 14 3 7 1 15 1 12 1 12 4 10 4 5 4 11 5 15 5 13 5 15 1 7 1 13 1 15 4 8 4 12 4 8 4 15 4 14 4 15 5 11 5 14 5 </p>
            </triangles>
         </mesh>
      </geometry>
      <geometry id="mesh2-geometry" name="mesh2-geometry">
         <mesh>
            <source id="mesh2-geometry-position">
               <float_array id="mesh2-geometry-position-array" count="48">-6.937500 0.023142 1.000000 7.062500 0.023142 1.000000 -6.937500 0.023142 2.000000 -6.937500 5.023142 1.000000 7.062500 0.023142 2.000000 7.062500 5.023142 1.000000 -0.489265 0.023142 2.000000 -6.937500 5.023142 2.000000 0.510735 0.023142 2.000000 7.062500 5.023142 2.000000 -0.489265 0.023142 7.000000 -0.489265 5.023142 2.000000 0.510735 0.023142 7.000000 0.510735 5.023142 2.000000 -0.489265 5.023142 7.000000 0.510735 5.023142 7.000000 </float_array>
               <technique_common>
                  <accessor source="#mesh2-geometry-position-array" count="16" stride="3">
                     <param name="X" type="float"/>
                     <param name="Y" type="float"/>
                     <param name="Z" type="float"/>
                  </accessor>
               </technique_common>
            </source>
            <vertices id="mesh2-geometry-vertex">
               <input semantic="POSITION" source="#mesh2-geometry-position"/>
            </vertices>
            <lines material="material_0_0" count="24">
               <input semantic="VERTEX" source="#mesh2-geometry-vertex" offset="0"/>
               <p>0 1 0 2 0 3 1 4 1 5 2 6 2 7 3 7 3 5 4 8 4 9 5 9 10 6 6 11 7 11 8 12 8 13 9 13 12 10 10 14 14 11 12 15 13 15 15 14 </p>
            </lines>
         </mesh>
      </geometry>
   </library_geometries>
   <library_cameras>
      <camera id="Camera-camera" name="Camera-camera">
         <optics>
            <technique_common>
               <perspective>
                  <xfov>46.666667</xfov>
                  <yfov>35.000000</yfov>
                  <znear>1.000000</znear>
                  <zfar>1000.000000</zfar>
               </perspective>
            </technique_common>
         </optics>
      </camera>
   </library_cameras>
   <library_visual_scenes>
      <visual_scene id="SketchUpScene" name="SketchUpScene">
         <node id="Model" name="Model">
            <node id="mesh1" name="mesh1">
               <instance_geometry url="#mesh1-geometry">
                  <bind_material>
                     <technique_common>
                        <instance_material symbol="material_0_16" target="#material_0_16ID"/>
                     </technique_common>
                  </bind_material>
               </instance_geometry>
            </node>
            <node id="mesh2" name="mesh2">
               <instance_geometry url="#mesh2-geometry">
                  <bind_material>
                     <technique_common>
                        <instance_material symbol="material_0_0" target="#material_0_0ID"/>
                     </technique_common>
                  </bind_material>
               </instance_geometry>
            </node>
         </node>
         <node id="Camera" name="Camera">
            <matrix>
               -0.987222 0.083170 -0.135921 -0.110277
               -0.159348 -0.515271 0.842083 35.884568
               0.000000 0.852982 0.521940 19.534892
               0.000000 0.000000 0.000000 1.000000
            </matrix>
            <instance_camera url="#Camera-camera"/>
         </node>
      </visual_scene>
   </library_visual_scenes>
   <scene>
      <instance_visual_scene url="#SketchUpScene"/>
   </scene>
</COLLADA>
\;
      close T;
      close PLANE;
}


__END__
asdfadfa


